#!/bin/bash

LOGGING=0

if [[ "${LOGGING}" == "1" ]] ; then
    exec 1>> "/Library/Logs/ivpn-dns.sh.logs" 2>&1
    
    DATE=`date`
    echo "${DATE}: $0 $@" 
fi

PATH=/usr/sbin:$PATH

PRI_IFACE=`echo 'show State:/Network/Global/IPv4' | scutil | grep PrimaryInterface | sed -e 's/.*PrimaryInterface : //'`
PSID=`echo 'show State:/Network/Global/IPv4' | scutil | grep PrimaryService | sed -e 's/.*PrimaryService : //'`

if [[ "${PRI_IFACE}" == "" ]]; then
    echo "Error: Primary interface not found"
    echo 
    exit 1
else
    echo "Primary interface: ${PRI_IFACE}"
    echo "PSID: ${PSID}"
    echo    
fi

function print_state {


    S_STATE=`echo "show State:/Network/Service/${PSID}/DNS" | scutil`
    S_SETUP=`echo "show Setup:/Network/Service/${PSID}/DNS" | scutil`

    echo "State: ${S_STATE}"
    echo 
    echo "Setup: ${S_SETUP}"
    echo
}

function is_dns_changed {    

    PREFIX=$1

    DNS_STATE=`echo "show ${PREFIX}:/Network/Service/${PSID}/DNS" | scutil`
    VPN_STATE=`echo "show State:/Network/IVPN/DNS" | scutil`

    if [[ "${DNS_STATE}" == "${VPN_STATE}" ]]; then
        return 1
    fi

    return 0
}

function is_vpn_dns_set {
    echo "show State:/Network/IVPN/Original/DNS/State" | scutil | grep ServerAddresses >/dev/null
}

function is_dns_set_by_ivpn {
    PREFIX=$1    
    echo "show ${PREFIX}:/Network/Service/${PSID}/DNS" | scutil | grep SetByIVPN >/dev/null
    return $?
}

function store_user_setting {
    PREFIX=$1

    echo "Storing ${PREFIX}:/ dns settings"

    if is_dns_set_by_ivpn "${PREFIX}"; then
        echo "Ignoring: Current DNS change was made by IVPN"
        return 1
    fi

    scutil <<_EOF
    d.init
    get ${PREFIX}:/Network/Service/${PSID}/DNS
    set State:/Network/IVPN/Original/DNS/${PREFIX}
_EOF
}

function restore_user_setting {
    PREFIX=$1

    echo "Restoring ${PREFIX}:/ dns settings"

    scutil <<_EOF
    d.init
    get State:/Network/IVPN/Original/DNS/${PREFIX}
    set ${PREFIX}:/Network/Service/${PSID}/DNS
_EOF
}

function update_setting {
    PREFIX=$1

    scutil <<_EOF
    d.init
    get State:/Network/IVPN/DNS
    set ${PREFIX}:/Network/Service/${PSID}/DNS
_EOF
}

function store_and_update {
    PREFIX=$1

    if [[ "${PREFIX}" == "" ]]; then
        return 127
    fi

    store_user_setting "${PREFIX}"
    update_setting "${PREFIX}"
}

if [ "$1" = "-up" ] ; then

    DOMAIN_NAME="ivpn-client"
    FOREIGN_OPTIONS=`env | grep -E '^foreign_option_' | sort | sed -e 's/foreign_option_.*=//'`

    while read -r option
    do
        case ${option} in
            *DOMAIN*)
                DOMAIN_NAME=${option//dhcp-option DOMAIN /}
                ;;
            *DNS*)
                VPN_DNS=${option//dhcp-option DNS /}
                ;;
        esac
    done <<< "${FOREIGN_OPTIONS}"

    echo "DOMAIN: $DOMAIN_NAME"
    echo "VPN DNS: $VPN_DNS"

    scutil <<_EOF
        d.init
        d.add ServerAddresses * ${VPN_DNS}
        d.add DomainName "${DOMAIN_NAME}"
        d.add SetByIVPN "true"

        set State:/Network/IVPN/DNS
_EOF

    store_and_update "Setup"
    store_and_update "State"

elif [ "$1" = "-update" ] ; then

    if ! is_vpn_dns_set ; then
        echo "Error: Cannot find original DNS configuration"
        exit 0
    fi

    sleep 2

    if is_dns_changed "Setup"; then
        echo "DNS Setup:/ changed. Updating...";
        store_and_update "Setup"
    fi;

    if is_dns_changed "State"; then
        echo "DNS State:/ changed. Updating...";
        store_and_update "State"
    fi;

elif [ "$1" = "-down" ] ; then

    # TODO: killall -HUP mDNSResponder

    if ! is_vpn_dns_set ; then
        echo "Error: Cannot find original DNS configuration"
        exit 0
    fi

    if ! is_dns_changed "State" ; then
        restore_user_setting "State"
    fi

    if ! is_dns_changed "Setup" ; then
        restore_user_setting "Setup"
    fi

    scutil <<_EOF
        remove State:/Network/IVPN/Original/DNS/Setup
        remove State:/Network/IVPN/Original/DNS/State
        remove State:/Network/IVPN/DNS

        quit
_EOF
else
    print_state
    exit 1
fi

